"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var postcss_1 = __importDefault(require("postcss"));
var rtlcss_1 = __importDefault(require("rtlcss"));
var _types_1 = require("../@types");
var _constants_1 = require("../constants");
var selectors_1 = require("../utilities/selectors");
var options_1 = require("../utilities/options");
exports.insertOverrideRules = function (appends, rule, options) {
    var ltrPrefix = options.ltrPrefix, rtlPrefix = options.rtlPrefix, source = options.source, processUrls = options.processUrls, useCalc = options.useCalc, stringMap = options.stringMap;
    var prefixes = source === _types_1.Source.ltr ? rtlPrefix : ltrPrefix;
    var ruleStr = rule.toString();
    var ruleFlippedtring = rtlcss_1.default.process(ruleStr, {
        clean: false,
        processUrls: processUrls,
        useCalc: useCalc,
        stringMap: options_1.getRTLCSSStringMap(stringMap)
    });
    if (ruleStr === ruleFlippedtring) {
        return;
    }
    var root = postcss_1.default.parse(ruleFlippedtring);
    var ruleFlipped = root.first;
    var index = 0;
    ruleFlipped.walk(function (nodeFlipped) {
        if (nodeFlipped.type === _constants_1.DECLARATION_TYPE) {
            var decl = rule.nodes[index];
            var declStr = decl.toString();
            var declFlippedStr = nodeFlipped.toString();
            if (declStr === declFlippedStr) {
                var prev = nodeFlipped.prev();
                if (prev && prev.type === _constants_1.COMMENT_TYPE) {
                    prev.remove();
                }
                nodeFlipped.remove();
            }
            else if (_constants_1.FLIP_PROPERTY_REGEXP.test(decl.prop.toString())) {
                var declClone = decl.clone();
                declClone.value = 'unset';
                nodeFlipped.before(declClone);
            }
        }
        index++;
    });
    selectors_1.addSelectorPrefixes(ruleFlipped, prefixes);
    appends.push({
        rule: rule,
        ruleLTR: source === _types_1.Source.ltr ? null : ruleFlipped,
        ruleRTL: source === _types_1.Source.rtl ? null : ruleFlipped
    });
};
